#!/usr/bin/perl
#
###########################################################################
#
# FORMMAILER
#
# Versendet Formulardaten aus einer HTML-Seite per Mail (Templatebasiert)
#
# Felder:
# name             normales Feld
# NUM_name         nur nummerische Werte
# EMPTY_name       "name" enthlt Wert von "EMPTY_name", falls "name" leer
# NNUL_name        Feld darf nicht leer sein
# 1LINE_name       Zeilenumbrche werden aus Feld gelscht (Header-Felder)
# MAIL_name        Mailadresse (optional)
# NNMAIL_name      Mailadresse (zwingend)
# IMG_name         Bilddatei (GIF/JPG/PNG) -> ungltige Dateien
#                  werden abgelehnt
# FILE_name        Datei mit beliebigem Format
#
# Verarbeitung im Template:
# <!--feldname-->  fgt Wert aus "feldname" ein
# <!--date-->      Fgt Datum im Mailheader-Format ein
# <!--msgid-->     generiert eine Message-ID und fgt diese ein
# <!--boundary-->  Part-Boundary fr MIME-Mails mit Attachment
#
# Formular:
# kann per GET oder POST versendet werden. Beim Upload von Dateien ist
# zwingend POST sowie ENCTYPE="multipart/form-data" erforderlich
# Im Formular mssen nebst den gewnschten Feldern noch zwei versteckte
# Felder eingebaut werden: okurl und notokurl, welche die Ziel-URL nach dem
# Versand (okurl) bzw. bei einem Fehler (notokurl) definieren. Die URLs
# mssen absolut, also mit Domain sein:
# <input type="hidden" name="okurl" value="http://www.server.ch/ok.htm"/>
#
###########################################################################
# Letzte nderung:        20. August 2005
# Autor:                  Jrg Sommer <jsommer@cgicorner.ch>
###########################################################################
use strict;
use CGI::Carp qw(fatalsToBrowser);
my (%FORM, $result, $mailDate, $mailDate2, @tmp);

my $template="simple.template";									# Template fr Formmailer
my $template2="";																# 2. Template fr Formmailer (optional)
my $mailVersand="TRUE";													# TRUE=sendmail / FALSE=mail.eml
my $sendmail="/usr/sbin/sendmail";							# Pfad zu sendmail

&readForm;																			# Formulardaten einlesen
&setMailTime;																		# Zeit fr Mail definieren
my $messageID="<$mailDate2.".&makeBoundary(14)."\@$ENV{HTTP_HOST}>";
my $boundary="------------".&makeBoundary(26);

$result=0;	# per Default kein Fehler

foreach (keys(%FORM)) {

	if ($FORM{$_} =~ m/\n/) {											# korrektur Zeilenumbrche
		$FORM{$_}=~ s/\r//g;
	} else {
		$FORM{$_}=~ s/\r/\n/g;
	}

	if (($_ =~ m/^NNUL_/) and ($FORM{$_} eq "")) {
		$result++;																	# NOT NULL Feld ist leer --> Fehler und Abbruch
		last;
	}
	if (($_ =~ m/^NUM_/) and ($FORM{$_} =~ m/[^0-9\.\, \-\']/)) {
		$result++;																	# NUMMERISCHES Feld enthlt Zeichen --> Fehler und Abbruch
		last;
	}
	if (($_ =~ m/^NNMAIL_/) and ($FORM{$_} eq "")) {
		$result++;																	# Mail (zwingend) Feld ist leer --> Fehler und Abbruch
		last;
	}
	if ($_ =~ m/^(NN)?MAIL_/) {										# Mailadresse --> Check ob gltig sonst Fehler und Abbruch
		if (($FORM{$_} !~ m/^[A-Z0-9\-\.\_]+\@[A-Z0-9\-\.\_]+\.[A-Z]{2,}$/i) and ($FORM{$_} ne "")) {
			$result++;
			last;
		}
	}
	if ($_ =~ m/^EMPTY_(.*)$/) {									# EMPTY-Feld --> Check ob Quellfeld leer und ggf. Wert einsetzen
		if ($FORM{$1} eq "") {
			$FORM{$1}=$FORM{"EMPTY_".$1};
		}
	}
	if ($_ =~ m/^IMG_/) {													# IMAGE-Feld --> Check ob gltig sonst Abbruch
		if (-e "tmp_".$_."_".$FORM{$_}) {						# Datei existiert
			@tmp=getImageSize("tmp_".$_."_".$FORM{$_});
			if (($tmp[0] < 1) or ($tmp[1] < 1)) {			# ungltiges Bild
				$result++;
				last;
			}
		}
	}
}

if ($result == 0) {
	&sendMail($template);
	&sendMail($template2) unless ($template2 eq "");
	&sendURL($FORM{"okurl"});
} else {
	&sendURL($FORM{"notokurl"});
}

foreach (keys(%FORM)) {													# hochgeladene Bilder wieder lschen
	if ($_ =~ m/^(IMG_|FILE_)/) {
		if (-e "tmp_".$_."_".$FORM{$_}) {
			unlink "tmp_".$_."_".$FORM{$_};
		}
	}
}


###########################################################################
# Subroutinen
###########################################################################
sub readForm {					# Parameter in %FORM einlesen
	my ($buffer,@pairs,@pair,$i);
	if ($ENV{'REQUEST_METHOD'} eq "GET") {
		$buffer = $ENV{'QUERY_STRING'};
	} else {
		binmode(STDIN);
		read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
	}
	if ($buffer =~ m/------------.*\n?.*Content/) {
		&getUploadData($buffer);
	} else {
		@pairs = split(/&/, $buffer);
		foreach (@pairs) {
			@pair = split(/=/, $_, 2);
			for ($i=0;$i<2;$i++) {
				$pair[$i] =~ tr/+/ /;
				$pair[$i] =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
			}
			$FORM{$pair[0]} = $pair[1];
		}
	}
}
###########################################################################
sub getImageSize {			# Bildgrsse anhand des Dateitypes bestimmen
	my ($width,$height);
	if ($_[0] =~ m/\.gif$/i) {
		($width,$height)=getImageSizeGIF($_[0]);
	} elsif ($_[0] =~ m/\.jpe?g$/i) {
		($width,$height)=getImageSizeJPG($_[0]);
	} elsif ($_[0] =~ m/\.PNG$/i) {
		($width,$height)=getImageSizePNG($_[0]);
	}
	return ($width,$height);
}
###########################################################################
sub getImageSizeGIF {		# Bildgrsse von GIF-Dateien bestimmen
	my ($lf,$image,$width,$height,$s,@size);
	$lf=$/;
	undef $/;
	open (IMAGE, $_[0]);
	binmode(IMAGE);
	$image=<IMAGE> ;
	close(IMAGE);
	$/=$lf;
	if(!(substr($image,0,6) =~ m/GIF8[7,9]a/) || (length($s=substr($image, 6, 4))!=4)) {
		return;
	}
	(@size)=unpack("C"x4,$s);
	$width= $size[1]<<8|$size[0];
	$height= $size[3]<<8|$size[2];
	return ($width,$height);
}
###########################################################################
sub getImageSizeJPG {		# Bildgrsse von JPG-Dateien bestimmen
	my ($lf,$image,$width,$height,@size);
	$lf=$/;
	undef $/;
	open (IMAGE, $_[0]);
	binmode(IMAGE);
	$image=<IMAGE> ;
	close(IMAGE);
	$/=$lf;
	my $count=2 ;
	my $length=length($image);
	my $ch="" ;
	while (($ch ne "\xda") && ($count<$length)) {
		while (($ch ne "\xff") && ($count<$length)) {
			$ch=substr($image,$count,1);
			$count++;
		}
		while (($ch eq "\xff") && ($count<$length)) {
			$ch=substr($image,$count,1);
			$count++;
		}
		if ((ord($ch) >= 0xC0) && (ord($ch) <= 0xC3)) {
			$count+=3;
			(@size)=unpack("C"x4,substr($image,$count,4));
			$width=$size[2]<<8|$size[3];
			$height=$size[0]<<8|$size[1];
			return ($width,$height);
		} else {
			(@size)= unpack("C"x2,substr($image,$count,2));
			$count += $size[0]<<8|$size[1];
		}
	}
}
###########################################################################
sub getImageSizePNG {		# Bildgrsse von PNG-Dateien bestimmen
	my ($lf,$image,$width,$height,@size);
	$lf=$/;
	undef $/;
	open (IMAGE, $_[0]);
	binmode(IMAGE);
	$image=<IMAGE> ;
	close(IMAGE);
	$/=$lf;
	if ($image =~ /IHDR(.{8})/) {
		$image = $1;
		($width,$height) = unpack( "NN", $image);
	} else {
		print "Datei ist keine gueltige PNG Datei\n";
	}
	return ($width,$height);
}
###########################################################################
sub getUploadData {			# MULTIPART-FORMDATA verarbeiten
	my ($status,@content,$name,$filename,$tmp);
	@content=split(/\n/,$_[0]);										# Zeilenweise aufsplitten
	$status=0;
	$name="";
	$filename="";
	foreach (@content) {													# bergebene Daten zeilenweise verarbeiten
		chomp;
		if ($status == 0) {													# Status 0 --> erwarte neue Eingaben / Eingaben in Datei speichern
			if ($_ =~ m/------------/) {							# Neues Feld
				if ($filename ne "") {									# $filename gesetzt --> Datei muss geschlossen werden.
					close(FILE);
				} else {
					$FORM{$name}=~ s/[\r\n]+$//g;					# normales Feld: Leerzeilen am Schluss entfernen
				}
				$status=1;															# setze Status auf 1
			} else {																	# vorhandene Daten in Datei schreiben
				if ($filename ne "") {									# Dateiname angegeben --> in Datei schreiben
					print FILE "$_\n"
				} else {																# kein Dateiname --> Feld ergnzen
					$FORM{$name}.="\n".$_;
				}
			}
		} elsif ($status == 1) {										# Status = 1 --> Name auslesen
			$_ =~ m/; name=\"(.*?)\"/;								# Name des Feldes auslesen
			$name=$1;
			$name=~ s/[\r\n]//g;
			if ($_ =~ m/filename=\"(.*?)\"/) {				# Dateiname bergeben?
				$filename=&fileNameOnly($1);						# Pfad wegschneiden
			} else {
				$filename=""; 													# Dateiname leer lassen
			}
			$status=2;																# setze Status auf 2
		} elsif ($status == 2) {										# leere Zeile --> weglassen
			$status = 3;
		} elsif ($status == 3) {										# Status = 3 --> Inhalt verarbeiten
			if ($filename eq "") {										# Dateiname leer
				$tmp=$_;
				$tmp=~ s/[\r\n]//g;
				$FORM{$name} = $tmp;										# --> normales Feld
			} else {																	# Dateiname gesetzt
				$filename =~ m/^([\w\/\-\_\. ]+)/;
				$filename=$1;
				$filename=~ m/^\s*(.*?)\s*$/g;
				open(FILE,"> tmp_".$name."_".$filename);					# Dateiinhalt --> Binrdatei erstellen
				binmode(FILE);
				$FORM{$name}=$filename;									# Dateiname in Variable setzen
			}
			$status=0;
		}
	}
}
###########################################################################
sub fileNameOnly {			# Gibt Dateiname zurck &fileNameOnly("Dateiname inkl. Pfad");
	my $tmp=$_[0];
	$tmp=~ m/^.*[\\\/](.*)$/;
	return $1;
}
###########################################################################
sub sendURL {						# &sendURL("http://redirect.to");
	my $url=$_[0];
	$url =~ s/&amp;/&/g;
	print "Location: $url\n\n";
}
###########################################################################
sub sendMail {
	$ENV{PATH} = "/bin:/usr/bin";
	$sendmail =~ m/([\w\/\-\_\.\\:]+)/;
	$sendmail=$1;
	if ($mailVersand eq "TRUE") {
		open MAIL, "| $sendmail -oi -t" or die "Sendmail konnte nicht geffnet werden";
	} else {
		open(MAIL,">mail-$_[0].eml") || die "Kann Versanddatei nicht schreiben! Error : $!\n";
	}

	open(TEMPLATE,$_[0]);
	while (<TEMPLATE>) {
		chomp;
		$_ =~ s/<!--date-->/$mailDate/gi;
		$_ =~ s/<!--msgid-->/$messageID/gi;
		$_ =~ s/<!--boundary-->/$boundary/gi;
		if ($_ =~ m/<!--((IMG|FILE)_.*?)-->/) {
			$tmp[0]=&attachment($1);
			$_ =~ s/<!--.*?-->/$tmp[0]/gi;
		}
		$_ =~ s/<!--(.*?)-->/$FORM{$1}/gi;
		print MAIL $_."\n";
	}
	close(TEMPLATE);

	close(MAIL);
}
###########################################################################
sub setMailTime {						# Zeit fr Mailversand setzen
	my (@dateValues,@engWochentag,@deuWochentag,@engMonate);
	@engWochentag = qw(Son Mon Tue Wed Thu Fri Sat);
	@engMonate = qw(&nbsp; Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	(@dateValues) = localtime(time);
	if ($dateValues[5] < 1900) { $dateValues[5]+=1900; }
	$dateValues[4]++;
	if ($dateValues[4] < 10) { $dateValues[4]="0".$dateValues[4]; }
	$dateValues[99]=$dateValues[3];
	if ($dateValues[3] < 10) { $dateValues[3]="0".$dateValues[3]; }
	if ($dateValues[2] < 10) { $dateValues[2]="0".$dateValues[2]; }
	if ($dateValues[1] < 10) { $dateValues[1]="0".$dateValues[1]; }
	if ($dateValues[0] < 10) { $dateValues[0]="0".$dateValues[0]; }
	$mailDate = $engWochentag[$dateValues[6]].", ".$dateValues[99]." ".$engMonate[$dateValues[4]]." ".$dateValues[5]." ".$dateValues[2].":".$dateValues[1].":".$dateValues[0];
	$mailDate2=$dateValues[5].$dateValues[4].$dateValues[3].$dateValues[2].$dateValues[1].$dateValues[0];
}
###########################################################################
sub makeBoundary {			# MIME Boundary
	my ($boundary, @boundaryv, $i);
	@boundaryv = (0..9, 'A'..'F');
	for ($i = 0; $i++ < $_[0];) {
		$boundary .= $boundaryv[rand(@boundaryv)];
	}
	return $boundary;
}
###########################################################################
sub attachment {				# sendet ein MIME-codiertes Attachment
	my $formName=$_[0];
	my $returnValue="";
	if (-e "tmp_".$formName."_".$FORM{$formName}) {
		$returnValue.="--$boundary\nContent-Type: application/octet-stream\nContent-Transfer-Encoding: base64\nContent-Disposition: attachment; filename=\"$FORM{$formName}\"\n\n";
		$returnValue.=&encodeBase64("tmp_".$formName."_".$FORM{$formName});
	}
	return $returnValue;
}
###########################################################################
sub encodeBase64 {			# &encodeBase64($file)
	open(FILE, $_[0]); binmode FILE;
	my $file = do{local$/;<FILE>};
	close(FILE);

	my ($res, $eol, $padding) = ("", "\n", undef);
	while (($file =~ /(.{1,45})/gs)) {
		$res .= substr(pack('u', $1), 1);
		chop $res;
	}
	$res =~ tr#` -_#AA-Za-z0-9+/#;
	$padding = (3 - length($_[0]) % 3) % 3;
	$res =~ s#.{$padding}$#'=' x $padding#e if $padding;
	$res =~ s#(.{1,76})#$1$eol#g if (length $eol);
	return $res;
}
